######################################################################################################################
#                                                                                                                    #
#                                                  ZIP utilities                                                     #
#                                                                                                                    #
######################################################################################################################

get_dirname_to_strip() {
	local striplevel=$1
	local baseDir=$2
	local dir2strip=.
	while [ $striplevel -gt 0 ]; do
		local subdir subdirname srcdirname
		for subdir in $baseDir/$dir2strip/* $baseDir/$dir2strip/.*; do
			local subdirname=$(basename $subdir)
			if [ "$subdirname" = "." -o "$subdirname" = ".." ]; then continue; fi
			if [ ! -d "$subdir" -o -n "$srcdirname" ]; then
				(cd $baseDir; ls -la $dir2strip) >&2
				error "Can only strip components in a zip if it contains only one directory per level to strip"
				return 4
			else
				srcdirname="$subdirname"
			fi
		done
		if [ -z "$srcdirname" ]; then
			error "Zip file seems to be empty..."
			return 4
		fi
		dir2strip="$dir2strip/$srcdirname"
		striplevel=$(expr $striplevel - 1)
	done
	echo $baseDir/${dir2strip#./}
}

zip_extract() {
	local srcFile="$1"
	local destDir="$2"
	local srcName=$(basename ${srcFile} .zip)
	
	log "Extracting sources into project..."
	if [ ${STRIP_LEVEL} -eq 0 ]; then
		log_exec mkdir -p ${baseDir}
		log_exec unzip -qq ${srcFile} -d ${baseDir}
	else
		# The zip, as usual, contain a directory, not directly files
		# We should move everything in the destination directory	
		local baseDir=$(dirname ${destDir})/${srcName}.tmp.$$
		log_exec mkdir -p ${baseDir}
		log_exec unzip -qq ${srcFile} -d ${baseDir}
		local dir2strip 
		if dir2strip=$(get_dirname_to_strip $STRIP_LEVEL $baseDir); then 
			cd $baseDir
			if [ ! -d ${destDir} ]; then
				log_exec mv $dir2strip ${destDir}
			else 
				# In case the directory is not empty...
				log_exec rm -rf ${destDir}/* ${destDir}/.[^.]* ${destDir}/..?*
				log_exec mv ${dir2strip}/* ${destDir}
				local file
				for file in ${dir2strip}/.[^.]* ${dir2strip}/..?*; do
					if [ -r "$file" ]; then
						log_exec mv ${file} ${destDir}
					fi
				done
			fi
			cd $destDir
			rm -rf ${baseDir}
		else
			rm -rf ${baseDir}
			fatal 4 "Can't extract zip file $srcName.zip"
		fi
	fi
}