#
# Functions and variables for the cpan class
#

# You can pass PERL flags by setting EXTRA_PERLFLAGS
# You can pass CPAN flags by setting EXTRA_CPAN_FLAGS


# Env var which tells perl if it should use host (no) or target (yes) settings
export PERLCONFIGTARGET="$(if $recipe_is_native; then echo no; else echo yes; fi)"

# Env var which tells perl where the perl include files are
export PERL_NATIVE_DIR="${NATIVE_SYSROOT_LOC}/usr/lib/perl-native/perl"
if $recipe_is_native; then
	export PERL_BASE="$PERL_NATIVE_DIR"
else
	export PERL_BASE="${SYSROOT_LOC}/usr/lib/perl"
fi

get_perl_version() {
	local base
	if [ -r "$PERL_BASE/config.sh" ]; then
		base="$PERL_BASE"
	elif [ -r "$PERL_NATIVE_DIR/config.sh" ]; then
		base="$PERL_NATIVE_DIR"
	else
		fatal 1 "Can't determine perl version"
	fi
	sed -n -e 's/^version='"'"'\([0-9]*\.[0-9]*\.[0-9]*\).*$/\1/p' $base/config.sh
}

export PERL_VERSION="$(get_perl_version)" 
export PERL_INC="${PERL_BASE}/${PERL_VERSION}/CORE"
export PERL_LIB="${PERL_BASE}/${PERL_VERSION}"
export PERL_ARCHLIB="${PERL_BASE}/${PERL_VERSION}"
export PERLHOSTLIB="${PERL_NATIVE_DIR}/${PERL_VERSION}/"

# Add LD_LIBRARY_PATH to find libperl.so
LD_LIBRARY_PATH="$PERL_INC:$LD_LIBRARY_PATH"
export LD_LIBRARY_PATH

cpan_do_configure () {
    export PERL5LIB="${PERL_ARCHLIB}"
    yes '' | nativeperl ${EXTRA_PERLFLAGS} Makefile.PL INSTALLDIRS=vendor ${EXTRA_CPANFLAGS}

    # Makefile.PLs can exit with success without generating a
    # Makefile, e.g. in cases of missing configure time
    # dependencies. This is considered a best practice by
    # cpantesters.org. See:
    #  * http://wiki.cpantesters.org/wiki/CPANAuthorNotes
    #  * http://www.nntp.perl.org/group/perl.qa/2008/08/msg11236.html
    [ -e Makefile ] || fatal 1 "No Makefile was generated by Makefile.PL"

    if ! $recipe_is_native; then
        . ${PERL_BASE}/config.sh
        # Use find since there can be a Makefile generated for each Makefile.PL
        for f in `find -name Makefile.PL`; do
            f2="${f%.PL}"
            test -f $f2 || continue
            log_exec sed -i "-e 's:\(PERL_ARCHLIB = \).*:\1${PERL_ARCHLIB}:'" \
                            "-e 's/perl.real/perl/'" \
                            "-e 's|^\(CCFLAGS =.*\)|\1 ${CFLAGS}|'" \
                            "$f2"
        done
    fi
}

cpan_do_compile () {
    do_make PASTHRU_INC="${CFLAGS}" # LD="${CCLD}"
}

cpan_do_install () {
    do_make DESTDIR="${installDir}" install_vendor
    for PERLSCRIPT in `grep -rIEl '#! *${bindir}/perl-native.*/perl' ${installDir}`; do
        log_exec sed -i -e 's|${bindir}/perl-native.*/perl|/usr/bin/env nativeperl|' $PERLSCRIPT
    done
}
