######################################################################################################################
#                                                                                                                    #
#                                                 Debugging help setup                                               #
#                                                                                                                    #
######################################################################################################################

log_utils_script_file=$(current_script_file)	# This is the current "log_utils.sh" script

: ${TERM:=dumb}
if [ "$TERM" = "dumb" ]; then
	eclipse_console=true
	for plugin in $sw4l_install_base/plugins/net.mihai-nita.ansicon.plugin_*; do
		if [ -r "$plugin" ]; then
			TERM="ansi"
			break
		fi
	done
else
	if stty -g > /dev/null 2>&1; then
		eclipse_console=true
	else
		eclipse_console=false
	fi
fi
export TERM

if $eclipse_console; then
	if [ -f "$HOME/.sw4l_colors" ]; then
		color_def="$HOME/.sw4l_colors"
	else
		color_def="$PLATFORM_LOC/.sw4l_colors"
	fi
else
	if [ -f "$HOME/.sw4l_colors-$TERM" ]; then
		color_def="$HOME/.sw4l_colors-$TERM"
	else
		color_def="$PLATFORM_LOC/.sw4l_colors-$TERM"
	fi
fi

if [ "$TERM" = "dumb" ]; then
	echo "NOTE:    If you are executing in an Eclipse console, think at installing the \"ANSI Escape in Console\""
	echo "NOTE:    plugin from 'https://www.mihai-nita.net/eclipse' to get coloring of your console output"
else
	if [ "$TERM" = "ansi" ] || [[ " $TERM" =~ " xterm" ]]; then
		use_ansi_sequences=true
	else
		use_ansi_sequences=false
	fi
	
	if [ $log_utils_script_file -nt $color_def ]; then
		if ! $eclipse_console; then
			cat << \FUNCS_EOF1 > $color_def
#
# Query a property from the terminal, e.g. background color.
#
# XTerm Operating System Commands
#     "ESC ] Ps;Pt ST"
#
# get_terminal_property [<property_no> (default: 11=text-background)]
#	This only works on a real (or virtual) terminal, not on an Eclipse console that has no /dev/tty
#
get_terminal_property() {
	if [ ! -r /dev/tty ]; then return; fi
	local oldstty
	if ! oldstty=$(stty -g 2>/dev/null); then return; fi
	
	# What to query?
	# 11: text background
	local Ps=${1:-11}
	
	stty raw -echo min 0 time 0
	# stty raw -echo min 0 time 1
	printf "\033]$Ps;?\033\\" > /dev/tty
	# xterm needs the sleep (or "time 1", but that is 1/10th second).
	sleep 0.1
	read -r answer < /dev/tty
	#echo $answer | cat -A
	result=${answer#*;}
	stty $oldstty
	# Remove escape at the end.
	echo $result | sed 's/[^rgb:0-9a-f/]\+$//'
}

use_dark_background() {
	if [ ! -r /dev/tty ]; then return 1; fi # Eclipse console has a white background (by default)
	local back_color=$(get_terminal_property 11)
	back_color=${back_color#*:}
	back_lumi=$(( 16#${back_color//\// + 16#} ))
	[ $back_lumi -le $(( 16#8000 )) ]
}
FUNCS_EOF1
		else
			cat << \FUNCS_EOF2 > $color_def
#
# Currently we don't know how to get the Eclipse theme so, as the dark theme
# is not so useful without a lot of customization, we just expect users to
# set an ECLIPSE_DARK_MODE environment variable to true if needed
#
: ${ECLIPSE_DARK_MODE:=false} # Set this to true in Platform Properties if needed

use_dark_background() {
	eval $ECLIPSE_DARK_MODE
}
FUNCS_EOF2
		fi
	
		if $use_ansi_sequences; then
			cat <<- \ANSI_EOF >> $color_def
	
				# color definitions
				export black="$(echo -e -n '\033[30m')"
				export red="$(echo -e -n '\033[31m')"
				export green="$(echo -e -n '\033[32m')"
				export yellow="$(echo -e -n '\033[33m')"
				export blue="$(echo -e -n '\033[34m')"
				export purple="$(echo -e -n '\033[35m')"
				export cyan="$(echo -e -n '\033[36m')"
				export white="$(echo -e -n '\033[37m')"
				export grey="$(echo -e -n '\033[90m')"
				export brightred="$(echo -e -n '\033[91m')"
				export brightgreen="$(echo -e -n '\033[92m')"
				export brightyellow="$(echo -e -n '\033[93m')"
				export brightblue="$(echo -e -n '\033[94m')"
				export brightpurple="$(echo -e -n '\033[95m')"
				export brightcyan="$(echo -e -n '\033[96m')"
				export brightwhite="$(echo -e -n '\033[97m')"
				export nc="$(echo -e -n '\033[0m')" # All attributes off
			ANSI_EOF
		else
			cat <<- \TPUT_EOF >> $color_def
	
				# color definitions
				export black="$(tput setaf 0)"
				export red="$(tput setaf 1)"
				export green="$(tput setaf 2)"
				export yellow="$(tput setaf 3)"
				export blue="$(tput setaf 4)"
				export purple="$(tput setaf 5)"
				export cyan="$(tput setaf 6)"
				export white="$(tput setaf 7)"
				export grey="$(tput setaf 8)"
				export brightred="$(tput setaf 9)"
				export brightgreen="$(tput setaf 10)"
				export brightyellow="$(tput setaf 11)"
				export brightblue="$(tput setaf 12)"
				export brightpurple="$(tput setaf 13)"
				export brightcyan="$(tput setaf 14)"
				export brightwhite="$(tput setaf 15)"
				export nc="$(tput sgr0)" # no color
			TPUT_EOF
		fi
		cat << \COLORS_EOF >> $color_def

# Colors for set -x prompt coloring
if $(use_dark_background); then
	export script_color="${brightblue}"
else
	export script_color="${blue}"
fi
export linenum_color="${red}"
export funcname_color="${green}"
export command_color="${nc}" # So that it's correct in both dark and light modes...

# Color for log messages
export debug_color="${green}"
if $(use_dark_background); then
	export note_color="${brightcyan}"
	export warning_color="${brightyellow}"
else
	export note_color="${cyan}"
	export warning_color="${yellow}"
fi
export error_color="${red}"
export internal_error_color="${purple}"
COLORS_EOF
	fi
	source $color_def
	export PS4='+ ${script_color}(${BASH_SOURCE##*/}${linenum_color}:${LINENO}${script_color})${funcname_color}${FUNCNAME[0]/source/}${script_color}: ${command_color}'
fi

######################################################################################################################
#                                                                                                                    #
#                                                   Debug utilities                                                  #
#                                                                                                                    #
######################################################################################################################

#
# To activate a trace channel just put its name in DEBUG_TRACES
#
: ${verbose_errors:=false}
active_debug_traces="$DEBUG_TRACES $active_debug_traces"
if [ -n $active_debug_traces ]; then
	active_debug_traces=" $active_debug_traces "
fi

log_backtrace() {
	local index="${1:-0}" 
	local count="${1:-999}"
	local linenum func script hd1 hd2 hdr
	local hd1="	At          "
	local hd2="	Called from "
	local hdr="$hd1"
	local spaces="            "
	while caller $((index++)); do 
		if [ $((--count)) -eq 0 ]; then 
			break
		fi
	done | while read linenum func script; do 
		echo "${hdr}${funcname_color}${func}${spaces:${#func}} ${script_color}${script}${nc}:${linenum_color}${linenum}${nc}"
		hdr="$hd2"
	done >&2
}

print_traces() {
	local trace
	for trace; do
		if [[ " $active_debug_traces " =~ " $trace " ]]; then continue; fi
		active_debug_traces="$active_debug_traces $trace"
	done
	active_debug_traces="${active_debug_traces# }"
}
for dbg_trace in $DEBUG_TRACES; do
	print_traces $dbg_trace
done
: ${DEBUG_CAT_MAX_LINES:=25}

noprint_traces() {
	local trace
	if [ -z "$*" ]; then
		active_debug_traces=
		return
	fi
	active_debug_traces=" ${active_debug_traces# } "
	for trace; do
		active_debug_traces="${active_debug_traces/ $trace / }"
	done
	active_debug_traces="${active_debug_traces% }"
	active_debug_traces="${active_debug_traces# }"
}

: ${PRINT_DEBUG_BACKTRACE:=false}
: ${debug_backtrace_print:=$PRINT_DEBUG_BACKTRACE}
dbg() {
	local trace_channel="$1"
	shift
	if [ -z $active_debug_traces ]; then return 0; fi
	if [[ " $active_debug_traces " =~ " $trace_channel " ]]; then
		echo "${debug_color}${trace_channel^^}:" "$@" "${nc}" >&2
		$debug_backtrace_print && log_backtrace 1 1
	fi
}

# Display a note if any debug channel is active
dbg_note() {
	if [ -z $active_debug_traces ]; then return 0; fi
	echo "${note_color}NOTE:    $*${nc}" >&2
}

dbg_cat() {
	local trace_channel="$1"
	local message="$2"
	local file="$3"
	local fsize
	if [ -z "$file" -a -r "$message" ]; then
		file="$message"
		message="Displaying"
	fi
	
	if [ -z $active_debug_traces ]; then return 0; fi
	if [[ " $active_debug_traces " =~ " $trace_channel " ]]; then
		echo "${debug_color}${trace_channel^^}: $message '$file':${nc}"
		if [ -s "$file" ]; then
			fsize=$(wc -l "$file")
			echo -n "${debug_color}"
			if [ ${fsize%% *} -gt $DEBUG_CAT_MAX_LINES ]; then
				head "$file"
				echo "${nc}        ........${debug_color}"
				tail "$file"
			else
				cat "$file"
			fi
			echo -n "${nc}"
		elif [ -r "$file" ]; then
			echo "${warning_color}	<empty>${nc}"
		else
			echo "${error_color}	<FILE NOT FOUND>${nc}"
		fi
		echo ""
		$debug_backtrace_print && log_backtrace 1 1
	fi >&2
}

trace_vars() {
	local trace_channel="$1"
	if [ -z $active_debug_traces ]; then return 0; fi
	if [[ " $active_debug_traces " =~ " $trace_channel " ]]; then
		shift
		local var
		for var; do
			echo "${debug_color}${trace_channel^^}: ${linenum_color}$var${nc}='${debug_color}${!var}${nc}'"
		done
		$debug_backtrace_print && log_backtrace 1 1
	fi
}

######################################################################################################################
#                                                                                                                    #
#                                                  Logging utilities                                                 #
#                                                                                                                    #
######################################################################################################################

: ${quiet:=false}
log() {
	$quiet || echo "$@" >&2
}

log_vars() {
	local var
	if $quiet; then return; fi
	for var; do
		echo "$var='${!var}'"
	done
	log_backtrace 1 1
}

internal_error() {
	local status="$1"
	shift
	local message="$*"
	if [ 0$status -ne 0 ]; then
		if [ -z "$message" ]; then
			message="status=$status"
			status=1
		else
			message="status=$status - $message"
		fi
	fi
	echo -n "${internal_error_color}INTERNAL ERROR: $message at " >&2
	log_backtrace 1
	exit $status
}

# Just display an ERROR message, possibly with a call backtrace
log_error() {
	local message="$*"
	echo "${error_color}ERROR:   $message${nc}" >&2
	if $verbose_errors; then
		log_backtrace 1
	fi
}

# This version is meant to be called from another logging function, not directly
__log_error() {
	local depth="$1"
	shift
	local message="$*"
	echo "${error_color}ERROR:   $message${nc}" >&2
	if $verbose_errors; then
		log_backtrace $((depth+1))
	fi
}

__fatal() {
	local depth="$1"
	shift
	local status="$1"
	shift
	local message="$*"
	if [ 0$status -ne 0 ]; then
		if [ -z "$message" ]; then
			message="status=$status"
			status=1
		else
			message="status=$status - $message"
		fi
	fi
	verbose_errors=true
	__log_error $((depth+1)) "$message"
	exit $status
}

# fatal <status> <message>
fatal() {
	__fatal 1 "$@"
}

# error <status> <message>
error() { # This is identical to fatal
	__fatal 1 "$@"
}

# warning <message>
warning() {
	echo "${warning_color}WARNING: $*${nc}" >&2
}

# __warning <status> <message>
__warning() {
	local status=$1
	shift
	echo "${warning_color}WARNING: $* [status=$status]${nc}" >&2
}

# message <message>
message() {
	echo "${note_color}NOTE:    $*${nc}" >&2
}

# note is a synonym for message (already defined in recipe_utils)
note() {
	echo "${note_color}NOTE:    $*${nc}" >&2
}

# fatal_no_status <message>
fatal_no_status() {
	__fatal 1 0 "$@"
}

# fatal_on_error <message>
fatal_on_error() {
	local status=$?
	if [  $status -ne 0 ]; then
		__fatal 1 "$@"
	fi
	return 0
}

# log_on_error <message>
log_on_error() {
	local status=$?
	if [  $status -ne 0 ]; then
		if [ -z "$message" ]; then
			message="status=$status"
			status=1
		else
			message="status=$status - $message"
		fi
		__log_error 1 "$message"
	fi
	return 0
}

# warn_on_error <message>
warn_on_error() { 
	local status=$?
	if [  $status -ne 0 ]; then
		warning $status "$@"
	fi
	return 0
}

######################################################################################################################
#                                                                                                                    #
#                                             Execution logging utilities                                            #
#                                                                                                                    #
######################################################################################################################

#
# Execute a command, logging it if needed, and exit in case of error
#
# For all variants we must be careful to requote arguments that contain
# spaces, but NOT prefixed environment variables or the command itself...
# Furthermore command arguments like XXX=... must be double quoted (to 
# expand variables) but those like --xxx=... should NOT be requoted
#
do_exec_locked() {
	local quiet=$1
	shift
	local lock=$1
	shift
	local cmd env sep opt 
	while [ $# -gt 0 ]; do
		cmd="$1"
		if echo "$cmd" | grep -s -q '^[a-zA-Z_][a-zA-Z_0-9]*='; then
			env="$env$sep$cmd"
			sep=" "
			shift
		else
			break
		fi
	done
	shift

	# Now we must be careful: passing parameters by "$@"
	# creates trailing empty parameters that we must suppress
	# as they may disturb the launched command, while if they
	# were intended they usually are meaningless, except when
	# they are followed by a non-empty parameter...
	declare -a opts
	declare -i n_empty=0
	for opt in "$@"; do
		if [ -n "$opt" ]; then
			while [ $n_empty -gt 0 ]; do
				let n_empty-=1
				opt+=( '' )
			done
			opts+=( $(requote_arg "${opt}") )
		else
			let n_empty+=1
		fi
	done
	$quiet || echo $env $cmd ${opts[*]} >&2
	if [ -n "$lock" ]; then
		flock $lock /bin/bash -c "$env $cmd ${opts[*]}"
	else
		eval "$env $cmd ${opts[*]}"
	fi
}

log_exec() {
	do_exec_locked $quiet '' "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		__fatal 1 $status "Command '$*' failed..." >&2
	fi
	return $status # Propagate, just in case...
}

log_exec_locked() {
	do_exec_locked $quiet "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		local lock="$1"
		shift
		if [ -n "$lock" ]; then
			lock="(locked on $lock) "
		fi
		__fatal 1 $status "Command '$*' ${lock}failed..." >&2
	fi
	return $status # Propagate, just in case...
}

silent_exec() {
	do_exec_locked true '' "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		__fatal 1 $status "Command '$*' failed..." >&2
	fi
	return $status # Propagate, just in case...
}

silent_exec_locked() {
	do_exec_locked true "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		local lock="$1"
		shift
		if [ -n "$lock" ]; then
			lock="(locked on $lock) "
		fi
		__fatal 1 $status "Command '$*' ${lock}failed..." >&2
	fi
	return $status # Propagate, just in case...
}

#
# Execute a command, logging it if needed, returning an error code
#
log_run() {
	do_exec_locked $quiet '' "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		__warning $status "Command '$*' failed..." >&2
	fi
	return $status
}

log_run_locked() {
	do_exec_locked $quiet "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		local lock="$1"
		shift
		if [ -n "$lock" ]; then
			lock="(locked on $lock) "
		fi
		__warning $status "Command '$*' ${lock}failed..." >&2
	fi
	return $status
}

silent_run() {
	do_exec_locked true '' "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		__warning $status "Command '$*' failed..." >&2
	fi
	return $status
}

silent_run_locked() {
	do_exec_locked true "$@"
	local status=$?
	if [ $status -ne 0 ]; then
		local lock="$1"
		shift
		if [ -n "$lock" ]; then
			lock="(locked on $lock) "
		fi
		__warning $status "Command '$*' ${lock}failed..." >&2
	fi
	return $status
}

#
# Execute a command, logging it if needed, not signaling errors, always succeeding
#
log_do() {
	do_exec_locked $quiet '' "$@"
	return 0
}

log_do_locked() {
	do_exec_locked $quiet "$@"
	return 0
}

silent_do() {
	do_exec_locked true '' "$@"
	return 0
}

silent_do_locked() {
	do_exec_locked true "$@"
	return 0
}
