#!/bin/bash
#
# Common Linux module class utilities (note module inherits from kernel)
#

#
# TODO: Have a reference to the kernel in a module package or the platform
# We may search for the reference kernel:
#	1)	as a dependency in the package
#	2)	as a dependency in the project (how? only for ac6-module projects?)
#	3)	by looking at all linux kernel projects in the platform, getting the latest
#	4)	by looking at all linux kernel projects in the workspace, getting the latest (how? only for ac6-kernel projects?)
# Currently only do 1 and 3
# 
search_reference_kernel() {
	if [ -n "$KDIR" -a -d "$KDIR" ]; then
		echo "$KDIR" 
		return 0
	fi
	
	local dep kernel found kernel_name
	for dep in $recipe_buildtime_dependencies; do
		for kernel in $platform_loc/metadata/kernels/$dep*/ac6_package_manifest.xml; do
			kernel_name="$(basename $(dirname $kernel))"
			if [ ! -d "../$kernel_name" ]; then continue; fi
			if grep -s -q 'typeId>fr.ac6.platform.c.kernel</typeId' $kernel; then
				found=$kernel_name
			fi
		done
	done
	if [ -n "$found" ]; then
		echo "../$found"
		return 0
	fi
	for kernel in $platform_loc/metadata/kernels/*/ac6_package_manifest.xml; do
		kernel_name="$(basename $(dirname $kernel))"
		if [ ! -d "../$kernel_name" ]; then continue; fi
		if grep -s -q 'typeId>fr.ac6.platform.c.kernel</typeId' $kernel; then
			found=$kernel_name
		fi
	done
	if [ -n "$found" ]; then
		echo "../$found"
		return 0
	fi
	return 1
}

kernel_build_tree=$(search_reference_kernel)
if [ -n "$kernel_build_tree" -a -d "$kernel_build_tree" ]; then
	export kernel_build_tree
	kernel_release=$(get_kernel_version "$kernel_build_tree")
	if [ -n "$kernel_release" ]; then
		export kernel_release
	fi
fi

# Add a few parameters to make to build relatively to the determined kernel
make_other_flags="$make_other_flags KERNEL_VERSION=$kernel_release KERNEL_DIR=$kernel_build_tree"

#
# Redefine do_install_module for module packages, importing or creating modules.* files
#
module_do_modules_install() {
	log "Copy Modules files"
	if false && is_in_class module; then # This is still buggy (and useless?)
		#
		# When installing an independent module, it would be nice to have
		# the module definition files coming from the kernel, like modules.builtin
		# and modules.order; however I don't have a variable giving the proper
		# kernel; that should be provided but would need to have the kernel
		# package name in the module name, so that we could still support several
		# kernels in the same platform...
		#
		# An alternative would be to compile and install automatically the module
		# against ALL kernels in the current platform...
		#
		# Anyway, waiting for a correct fix, we just disable this code.
		#
		if [ -n "$kernel_release" ]; then
			modulesDir="lib/modules/$kernel_release"
			log_exec install -d "$installDir/$modulesDir"
			if [ -d "kernel-build-tree/_install/$modulesDir" ]; then
				log_exec install "kernel-build-tree/$modulesDir"/modules.* "$installDir/$modulesDir"
			else
				# Don't find installed modules, just pretend there is none
				touch "$installDir/$modulesDir/modules.order"
				touch "$installDir/$modulesDir/modules.builtin"
			fi
		fi
	fi
	do_make_modules_install "$@"
}

# Compatibility rotines
module_do_modules_install() {
	module_do_modules_install "$@"
}

module_do_install() {
	do_make_modules_install "$@"
}



