######################################################################################################################
#                                                                                                                    #
#                                                  PACKAGE configuration                                             #
#                                                                                                                    #
######################################################################################################################

# Package-specific variables
package_id=$recipe_id
package_name=$recipe_name
package_version=$recipe_version
package_revision=$recipe_revision
package_arch=${PACKAGE_ARCH//_/-/}

# Default values
: ${package_arch:=all}

# 
#                              ######################
#                              ######################
#                              ##                  ## 
#                              ##    HACK ALERT    ## 
#                              ##                  ## 
#                              ######################
#                              ######################
#
#
# Currently we ONLY generate installable packages for target elements, but we may have
# target-installable elements generated by -cross packages (which are a special kind
# of native packages, generating elements going in native and target sysroots, the
# elements going in the target sysroot having to be packaged in the target feeds
#
# For this to work without heavily modifying the split/package steps, we always
# generate package tarballs, ipk and metadata in the target feeds, even for a native
# package (as only target-installable data will be placed there).
#
# This may have to be corrected to fully support cross or SDK packages
#
# Waiting for a better solution (a PLATFORM_ARCH variable for example), we start from
# the compiler prefix, hoping the relation with the target architecture will be stable...
#
if [ "$package_arch" = "native" ]; then
	package_arch=${arch_compiler_prefix#*-}
	package_arch=${package_arch%%-*}
	package_arch=${package_arch//_/-}
fi

# Package-specific derived variables
package_suffix=${recipe_version}-${recipe_revision}_${package_arch}

######################################################################################################################
#                                                                                                                    #
#                                              Layout of the feeds directory                                         #
#                                                                                                                    #
######################################################################################################################

#
#	feeds
#		contents
#			$arch_dir_name
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#			native
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#			all
#				$recipe_id
#					$bin_package.list			// List of (absolute path names of) all files contained in the package
#		metadata
#			$arch_dir_name
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					$bin_package.rdepends
#					$bin_package.rrecommends
#					$bin_package.rsuggests
#					$bin_package.rconflicts
#					$bin_package.rreplaces
#					$bin_package.summary
#					$bin_package.description
#					$bin_package.section
#					$bin_package.homepage
#					$bin_package.license
#					$bin_package.preinst
#					$bin_package.postinst
#					$bin_package.prerm
#					$bin_package.postrm
#			native
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					...
#			all
#				$recipe_id
#					$recipe_id.manifest			// List of all binary packages generated by this recipe
#					$bin_package.auto_depends	// Metadata of the binary package
#					...
#		packages
#			$arch_dir_name
#				$bin_package					// ID of the generating recipe
#			native
#				$bin_package					// ID of the generating recipe
#			all
#				$bin_package					// ID of the generating recipe
#		tarballs
#			$arch_dir_name
#				$recipe_id
#					$bin_package.tar.bz2		// Installable binary package tarball (without metadata)
#			native
#				$recipe_id
#					$bin_package				// Installable binary package tarball (without metadata)
#			all
#				$recipe_id
#					$bin_package				// Installable binary package tarball (without metadata)
#		ipk
#			$arch_dir_name
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#			native
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#			all
#				$recipe_id
#					$bin_package.ipk			// Installable binary package (with metadata)
#

feedsDir=$PLATFORM_LOC/feeds

# Sub-directories where to place generated files
metadataSubDir=metadata/$package_arch/$package_id
contentsSubDir=contents/$package_arch/$package_id
packagesSubDir=packages/$package_arch
tarballsSubDir=tarballs/$package_arch/$package_id
ipkSubDir=ipk/$package_arch

# Package-dependent feed directories
metadataDir=${feedsDir}/$metadataSubDir
contentsDir=${feedsDir}/$contentsSubDir
packagesDir=${feedsDir}/$packagesSubDir
tarballsDir=${feedsDir}/$tarballsSubDir
ipkDir=${feedsDir}/$ipkSubDir

# Package-independent feed sub-directories
allMetadataSubDir=metadata/$package_arch
allContentsSubDir=contents/$package_arch
allPackagesSubDir=packages/$package_arch
allTarballsSubDir=tarballs/$package_arch
allIpkSubDir=ipk/$package_arch

# Package-independent feed directories
allMetadataDir=${feedsDir}/$allMetadataSubDir
allContentsDir=${feedsDir}/$allContentsSubDir
allPackagesDir=${feedsDir}/$allPackagesSubDir
allTarballsDir=${feedsDir}/$allTarballsSubDir
allIpkDir=${feedsDir}/$allIpkSubDir

# Architecture-independent feed directories
baseMetadataDir=${feedsDir}/metadata
baseContentsDir=${feedsDir}/contents
basePackagesDir=${feedsDir}/packages
baseTarballsDir=${feedsDir}/tarballs
baseIpkDir=${feedsDir}/ipk

######################################################################################################################
#                                                                                                                    #
#                                                 Package-name utilities                                             #
#                                                                                                                    #
######################################################################################################################

# Splits a full package name in 
#	package-name
#	version
#	revision
#	architecture
# separated by the provided character (colon by default)
split_package_name() {
	local separator="${2:-:}"
	full_package_name "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)_\(.*\)$/\1'$sep'\2'$sep'\3'$sep'\4/'
}

# Splits a package id in 
#	package-name
#	version
#	revision
# separated by the provided character (colon by default)
split_package_id() {
        local separator="${2:-:}"
        echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)$/\1'$sep'\2'$sep'\3/'
}

# Returns a package ID from a package name
package_id() {
	echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\).*$/\1_\2-\3/'
}

# Returns a package architecture from a package name
package_arch() {
	echo "$1" | sed -e 's/^\([^_]\+\)_\([^-_]\+\)-\([^_]\+\)\_\(.*\)$/\4/'
}

# Compute the name of a package-specific variable
package_var_name() {
	local package=${1%%_*}
	package=${package//-/_}
	package=${package//+/P}
	local variable="$2"
	echo ${variable}_${package}
}

# Get the value of a package-specific variable
# Here I may add support for overrides, if it becomes needed...
package_var() {
	local name=$(package_var_name "$@")
	local prepend="${name}_prepend"
	local append="${name}_append"
	echo ${!prepend}${!name}${!append}
}

package_contents() {
	local pkg="$1"
	local contents="$(echo ${contentsDir}/${pkg}*.list)"
	if [ -r "$contents" ]; then 
		echo "$contents"
	fi
}

full_package_name() {
	local pkg="$1"
	local needed="$2"
	local contents="$(package_contents $pkg)"
	if [ -n "$contents" ]; then
		basename "$contents" .list
	elif [ -n "$needed" ]; then
		echo "Package '$pkg' was not built" >&2
		return 1
	fi
}

package_depends() {
	local pkg="$1"
	local how="${2-RDEPENDS}"
	(
		local dep auto_dep=${how,,}
		local depends=$(echo ${metadataDir}/$pkg*/$pkg*.auto-${auto_dep#r})
		if [ -s "$depends" ]; then
			for dep in $(cat "$depends"); do
				echo $dep
			done
		fi
		for dep in $(package_var $pkg ${how^^}); do
			echo $dep
		done
	) | sort | uniq
}

######################################################################################################################
#                                                                                                                    #
#                                                 Pkg-config generation                                              #
#                                                                                                                    #
######################################################################################################################

pc_tag_vars="
	Name:recipe_name
	Description:SUMMARY
	Version:recipe_version_revision
	Requires:recipe_buildtime_dependencies
	Conflicts:RCONFLICTS
"

# If there are specific libraries to include, add their names to pc_additional_libs
# If specific C flags must be provided, specify them in pc_additional_cflags
# You don't need to add -L or -I for your package, they are automatically provided
pc_additional_libs=
if [[ " $recipe_name " =~ " lib*" ]]; then
	pc_additional_libs=$recipe_name
fi
pc_additional_cflags=

#
# Generate a .pc file for the current recipe
#
# If the recipe has defined non-standard installation directories that may be used
# in Cflags or Libs, give their names in $non_standard_install_dir_vars
#
# By default, if a library was created having the name of the recipe, a -l link option is generated
# If additional libraries must also be linked in depending packages, define them in $pc_additional_libs
#
# If additional variables must be included in the .pc file
# their NAMES must be provided as parameters
#

generate_pc_file_vars() {
	local vars_file="$1"
	shift
	local all_vars="$*"
	local var value exec_values data_values
	
	for var in $all_vars; do
		value="${!var}"
		if [ -z "$value" ]; then continue; fi
		case $var in
			*bindir | *libdir | *libexecdir)
#				if [ ! -d "_install/$value" ]; then continue; fi 
				tac $vars_file.exec.sed > $vars_file.exec.rev.sed
				echo "$var=$value" | sed -f $vars_file.exec.rev.sed
				if [[ " $exec_values " =~ " $value " ]]; then continue; fi
				exec_values="$exec_values $value"
				echo '\|^'"$var"'=|!s|'"$value/"'|${'"$var"'}/|' >> $vars_file.exec.sed.rev
				echo '\|^'"$var"'=|!s|'"$value"'$|${'"$var"'}|' >> $vars_file.exec.sed.rev
				;;
			*dir)
#				if [ ! -d "_install/$value" ]; then continue; fi 
				tac $vars_file.data.sed > $vars_file.data.rev.sed
				echo "$var=$value" | sed -f $vars_file.data.rev.sed
				if [[ " $data_values " =~ " $value " ]]; then continue; fi
				data_values="$data_values $value"
				echo '\|^'"$var"'=|!s|'"$value/"'|${'"$var"'}/|' >> $vars_file.data.sed
				echo '\|^'"$var"'=|!s|'"$value"'$|${'"$var"'}|' >> $vars_file.data.sed
				;;
			*)
				echo "$var=$value"
				;;
		esac
	done
}

generate_pc_file() {
	local pc_file="${1:-$recipe_name}.pc"
	local pc_dir="_install/usr/lib/pkgconfig"
	local pkg_var=$(shell_variable_name $recipe_name)
	local vars_file=$tmpdir/all_pc_vars.$$
	local additional_vars="$*"
	local var var_name spec tag spec_var value val normalized_value lib
	
	# First take into account the case where the pc_file passed is absolute
	case $pc_dir in
		/*)	pc_dir=_install${pc_file%/*}
			pc_file=${pc_file##*/}
			;;
	esac
	mkdir -p "$pc_dir"
	pc_file="$pc_dir/$pc_file"
	rm -f $vars_file.*

	for var in prefix exec_prefix; do
		echo '\|^'"$var"'=|!s|'"${!var}/"'|${'"$var"'}/|'
		echo '\|^'"$var"'=|!s|'"${!var}"'$|${'"$var"'}|'
	done >> $vars_file.exec.sed
	for var in exec_prefix prefix; do
		echo '\|^'"$var"'=|!s|'"${!var}/"'|${'"$var"'}/|'
		echo '\|^'"$var"'=|!s|'"${!var}"'$|${'"$var"'}|'
	done >> $vars_file.data.sed
	
	(
		# Now generate the file
		echo "# Standard directories" 
		echo "prefix=${prefix}" 
		if [ "$prefix" = "$exec_prefix" ]; then
			echo "exec_prefix=\${prefix}" 
		else
			value=" $exec_prefix"
			value=${value/ $prefix\// \$prefix/}
			value=${value# }
			echo "exec_prefix=${value}" 
		fi
		generate_pc_file_vars $vars_file $install_dir_vars
		
		if [ -n "$non_standard_install_dir_vars" ]; then
			echo # Separate standard and non-standard variables 
			echo "# Nonstandard directories"
			generate_pc_file_vars $vars_file $non_standard_install_dir_vars
		fi
		
		if [ -n "$additional_vars" ]; then
			echo # Separate standard and non-standard variables 
			echo "# Additional variables"
			generate_pc_file_vars $vars_file $additional_vars
		fi
	
		echo # Also separate description from variables
		echo "# Package description"
		for spec in $pc_tag_vars; do
			var=$(shell_variable_name ${spec#*:})
			tag=${spec%%:*}
			value=
			var_name=$var
			if [ "${!var_name+set}" ]; then
				value="${!var_name}"
			else
				var_name=$(shell_variable_name ${var}_${recipe_name})
				if [ "${!var_name+set}" ]; then
					value="${!var_name}"
				fi
			fi
			if [ -n "${value}" ]; then
				if [[ " $var " =~ "_dependencies " || " $var " =~ " R" ]]; then
					normalized_value=
					for val in $value; do
						case $val in
							\(*)	
								normalized_value="$normalized_value ${val#(}"
								;;
							*\))	
								normalized_value="$normalized_value ${val%)}"
								;;
							*_*)	
								normalized_value="$normalized_value ${val%_*} >= ${val#*_}"
								;;
							*)
								normalized_value="$normalized_value ${val}"
								;;
						esac
					done
					echo "${tag}:${normalized_value}"
				else
					echo "${tag}: ${value}"
				fi
			fi
		done
		
		# Adds the includedir to the Cflags
		if [ -n "$includedir" -a -d "_install/$includedir" ]; then
			normalized_value="-I\${includedir}"
			if [ -n "pc_additional_cflags" ]; then
				normalized_value="$normalized_value $pc_additional_cflags"
			fi
		else
			normalized_value="$pc_additional_cflags"
		fi
		if [ -n "$normalized_value" ]; then
			echo "Cflags: ${normalized_value# }"
		fi
	
		# Adds the libdir to the Libs and normalize library references
		if [ -n "$libdir" -a -d "_install/$libdir" ]; then
			normalized_value="-L\${libdir}"
		fi
		for lib in $recipe_name $pc_additional_libs; do
			lib=${lib#lib}
			lib=${lib%.a}
			lib=${lib%.so}
			value="$(find _install -name lib${lib}.so)$(find _install -name lib${lib}.a)"
			if [ -n "$value" ]; then
				normalized_value="$normalized_value -l${lib}"
			fi
		done
		if [ -n "$normalized_value" ]; then
			echo "Libs: ${normalized_value# }"
		fi
	) >> $pc_file
	rm -f $vars_file.*
}

######################################################################################################################
#                                                                                                                    #
#                                     Libtool and DT_RPATH/DT_RUNPATH management                                     #
#                                                                                                                    #
######################################################################################################################

#
# Find all ELF files in a directory tree
#
find_ELF_files() {
	local dir="$1"
	if [ -z "$dir" -o ! -d "$dir" ]; then return 0; fi
	find $dir -type f -exec file '{}' + \
	| sed -n -e 's/^\(..*\):[[:space:]][[:space:]]*ELF[[:space:]].*$/\1/p'
}

patch_sysroot_RPATH_values=false
adjust_libtool_scripts_and_rpath_for_sysroot() {
	local lock
	if [ X"$1" = X"-l" ]; then
		lock=$2
		shift 2
	fi

	local staging_dir="$1"
	local sysroot_dir="$2"
	if [ -z "$staging_dir" -o ! -d "$staging_dir" ]; then
		warning "No staging dir to adjust; bail out"
		return 0
	fi
	if [ -z "$sysroot_dir" ]; then
		warning "Target sysroot is not defined; bail out"
		return 0
	fi
	
	local ifile nb_rm=0
	declare -a to_rm
	
	#
	# We populate the target sysroot for a package, libtool scripts
	# and binaries may have to be adjusted to refer to the sysroot and not
	# the target rootfs; to get best result we first simplify all entries
	# then prefix them by the sysroot dir path
	#
	# Note that we must not modify files in-place but create and install a
	# copy as we work on a shallow copy (all files are hard-links to the
	# original file from the install step); furthermore we must lock the
	# install directory when deleting copied files, as the mere fact to 
	# remove a hard link will change the file inode and may disturb tar 
	# when splitting the package output... For efficiency we will do the
	# effective remove only at the end of the function
	#
	
	# For libtool files, they should be correctly referencing the native sysroot,
	# so we do not have to adjust them... However target libtool files were
	# sometimes (often/always?) generated for the target rootfs, not the target
	# sysroot, so we must adjust them...
	#
	# The last substitution may remove the sysroot_dir and keep the = or remove
	# the = and keep the sysrootdir; the former expects libtool to be called with
	# the sysroot and allow it to be relocated while the latter is safer but does
	# no more allow the sysroot to be relocated...
	#
	# Note there may be a problem if we have libtool-references to shared objects
	# in the build tree, that should reference the sysroot instead. This really 
	# should be handled by libtool at install time; it is too late now
	# as we don't know where (and if) these files will be in the sysroot...
	log "Patching libtool files to match their installed location"
	for ifile in $(find $staging_dir -type f -name '*.l[ao]'); do
		sed -e 's|\([[:space:]'"'"'=]\)'"$sysroot_dir"'/|\1/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/usr/|\1/usr/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/opt/|\1/opt/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/share/|\1/share/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/bin/|\1/bin/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/sbin/|\1/sbin/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/lib/|\1/lib/|g' \
		    -e 's|\([[:space:]'"'"'=]\)'"$sysroot_dir"'$|\1/|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/usr$|\1/usr|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/opt$|\1/opt|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/share$|\1/share|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/bin$|\1/bin|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/sbin$|\1/sbin|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/lib$|\1/lib|g' \
		    -e 's|\([[:space:]'"'"'=]\)/|\1'"$sysroot_dir"'/|g' \
		    -e 's|='"$sysroot_dir"'|=|g' \
		    "$ifile" > "$ifile.new"
		if cmp -s "$ifile" "$ifile.new"; then
			rm -f "$ifile.new"
		else
			mv "$ifile" "$ifile.old"
			to_rm[nb_rm++]="$ifile.old"
			mv "$ifile.new" "$ifile"
			log "Patched '${ifile#$staging_dir}'"
		fi
	done
	
	# We should also adjust the DT_RPATH/DT_RUNPATH values in the libraries
	# and binaries; however:
	#	1)	binaries are not needed in the target sysroot (and may even be 
	#		eliminated)
	#	2)	libraries are only searched for by the linker that will use
	#		a.	either the -L directives retargeted to the sysroot by the 
	#			cross_gcc scripts
	#		b.	or the -rpath directives provided by the libtool files we've
	#			patched above
	#	3)	native binaries and libraries are including RPATH values that were
	#		referencing directly the native sysroot, so don't have to be adjusted
	# So here we don't really have to patch DT_PATH/DT_RUNPATH; we in fact may
	# just eliminate all references outside the target sysroot. To try keeping 
	# the DT_RPATH/DT_RUNPATH short, and to allow moving the sysroot tree, we
	# could convert all path in the sysroot to be relative to the including file;
	# however the benefit is minimal, so we may as well skip this.
	#
	# Note there may be a problem if we have rpath-references to shared objects
	# in the build tree, that should reference the sysroot instead. This really 
	# should be handled at build/install time; it is too late at populate-sysroot
	# time as we don't know where (and if) these files will be in the sysroot...
	local old_rpath new_rpath idir iroot
	log "Search for ELF files from '$staging_dir' in which to patch DT_RPATH/DT_RUNPATH"

	for ifile in $(find_ELF_files $staging_dir); do
		old_rpath=$(chrpath -l "$ifile" 2> /dev/null)
		if [ $? -ne 0 ]; then
			# No DT_RPATH nor DT_RUNPATH, can't (and don't need to) add one...
			continue
		fi
		old_rpath=$(echo "$old_rpath" | sed -e 's|^'"${ifile}"':.*PATH=||')
		if [ "$old_rpath" = '$ORIGIN' ]; then
			# Simple relocatable RPATH, keep it as is
			continue
		fi
		
		idir=$(dirname $ifile | sed -e "s:$staging_dir:$sysroot_dir:")
		new_rpath=
		path_list="${old_rpath//:/ }"
		for opath in $path_list; do
			if [ -z "$opath" ]; then continue; fi
			if [[ " $opath" =~ " /" ]]; then 
				# Relative paths are meaningless (if they exist...)
				if $patch_sysroot_RPATH_values; then
					npath=$(get_relative_path "$opath" "$idir")
					if [ "${npath#/}" = "." ]; then
						opath="\$ORIGIN"
					else
						opath="\$ORIGIN/${npath#/}"
					fi
				fi
				new_rpath="${new_rpath}:${opath}"
			fi
		done
		new_rpath="${new_rpath#:}"
		
		if [ "$sysroot_dir" = "$TARGET_SYSROOT_LOC" -a ${#new_rpath} -gt ${#old_rpath} ]; then
			# Don't complain if not being able to retarget target code RPATH; we may in fact
			# completely skip this part for target binaries but it's better, for cases where
			# the RPATH references the target sysroot to convert it to a relative path so that
			# there does not remain references to the build environment (we could also patch
			# them to reference the target rootfs, so that all binaries will be similar).
			continue
		fi
		if [ "$old_rpath" != "$new_rpath" ]; then
			# Beware that the file may be shared with the install tree or the
			# other sysroot, through hard links, so we must make a copy first
			cp "$ifile" "$ifile.new" 
			if chrpath -r "$new_rpath" "$ifile.new" > /dev/null; then
				mv "$ifile" "$ifile.old.$$"
				to_rm[nb_rm++]="$ifile.old.$$"
				mv "$ifile.new" "$ifile"
				log "Patched rpath in '${ifile#$staging_dir}' from '$old_rpath' to '$new_rpath'"
			else
				rm "$ifile.new"
				warning "Failed to patch rpath in '${ifile#$staging_dir}' from '$old_rpath' to '$new_rpath'"
			fi
		fi
	done
	if [ $nb_rm -gt 0 ]; then
		if [ -n "$lock" ]; then
			silent_run_locked "$lock" rm -f "${to_rm[@]}"
		else
			silent_run rm -f "${to_rm[@]}"
		fi
	fi
}

adjust_libtool_scripts_and_rpath_for_packaging() {
	local lock
	if [ X"$1" = X"-l" ]; then
		lock=$2
		shift 2
	fi
	local staging_dir="${1:-.}"
	local ifile nb_rm=0
	declare -a to_rm
	
	# We create a target package that will run on the target board, libtool
	# scripts and binaries may have to be adjusted to refer to the target
	# rootfs and not the target sysroot, although this is only relevant for
	# -dev packages that are usually not installed on the target system.
	# For completeness we also simplify anything that look like a sysroot
	# or build-system internal reference
	#
	# Note that we must not modify files in-place but create and install a
	# copy as we work on a shallow copy (all files are hard-links to the
	# original file from the install step); furthermore we must lock the
	# install directory when deleting copied files, as the mere fact to 
	# remove a hard link will change the file inode and may disturb tar 
	# when populating the sysroot... For efficiency we will do the
	# effective remove only at the end of the function
	
	# For libtool files, patching is seldom needed as development packages
	# in which they are included are usually never installed on the target
	# system, but who cares...
	for ifile in $(find $staging_dir -type f -name '*.l[ao]'); do
		sed -e 's|\([[:space:]'"'"'=]\)/home/..*/usr\([/:[:space:]'"'"']\)|\1/usr\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/opt\([/:[:space:]'"'"']\)|\1/opt\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/share\([/:[:space:]'"'"']\)|\1/share\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/bin\([/:[:space:]'"'"']\)|\1/bin\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/sbin\([/:[:space:]'"'"']\)|\1/sbin\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/lib\([/:[:space:]'"'"']\)|\1/lib\2|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/usr$|\1/usr|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/opt$|\1/opt|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/share$|\1/share|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/bin$|\1/bin|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/sbin$|\1/sbin|g' \
		    -e 's|\([[:space:]'"'"'=]\)/home/..*/lib$|\1/lib|g' \
		    "$ifile" > "$ifile.new"
		if cmp -s "$ifile" "$ifile.new"; then
			rm -f "$ifile.new"
		else
			mv "$ifile" "$ifile.old.$$"
			to_rm[nb_rm++]="$ifile.old.$$"
			mv "$ifile.new" "$ifile"
			log "Patched '${ifile#$staging_dir}'"
		fi
	done
	
	# We must also adjust the DT_RPATH/DT_RUNPATH values in the libraries
	# and binaries; here we must retarget all DT_RPATH/DT_RUNPATH references
	# so that they refer directly to the target filesystem
	#
	# Note that the RPATH generated by the cross-compiler tools should already
	# be correct, as the compiler wrapper script generates target-oriented 
	# absolute -rpath directives, so we should not have to patch anything here.
	local old_rpath new_rpath
	log "Search for ELF files from '$staging_dir' in which to patch DT_RPATH/DT_RUNPATH"
	for ifile in $(find_ELF_files $staging_dir); do
		old_rpath=$(chrpath -l "$ifile" 2> /dev/null)
		if [ $? -ne 0 ]; then
			# No DT_RPATH nor DT_RUNPATH, can't (and don't need to) add one...
			continue
		fi
		old_rpath=$(echo "$old_rpath" | sed -e 's|^'"${ifile}"':.*PATH=||')
		if [ "$old_rpath" = '$ORIGIN' ]; then
			# Relocatable RPATH, keep it as is
			continue
		fi
		new_rpath=$(echo ":${old_rpath}:" | \
					sed -e 's|:/home/[^:][^:]*/usr\([/:]\)|:/usr\1|g' \
						-e 's|:/home/[^:][^:]*/opt\([/:]\)|:/opt\1|g' \
						-e 's|:/home/[^:][^:]*/share\([/:]\)|:/share\1|g' \
						-e 's|:/home/[^:][^:]*/bin\([/:]\)|:/bin\1|g' \
						-e 's|:/home/[^:][^:]*/sbin\([/:]\)|:/sbin\1|g' \
						-e 's|:/home/[^:][^:]*/lib\([/:]\)|:/lib\1|g' \
					)
		new_rpath=${new_rpath#:}
		new_rpath=${new_rpath%:}
		if [ "$old_rpath" != "$new_rpath" ]; then
			# Beware that the file may be shared with the install tree or the
			# other sysroot, through hard links, so we must make a copy first
			cp "$ifile" "$ifile.new" 
			if chrpath -r "$new_rpath" "$ifile.new" > /dev/null; then
				mv "$ifile" "$ifile.old.$$"
				to_rm[nb_rm++]="$ifile.old.$$"
				mv "$ifile.new" "$ifile"
				log "Patched rpath in '${ifile#$staging_dir}' from '$old_rpath' to '$new_rpath'"
			else
				rm "$ifile.new"
				warning "Failed to patch rpath in '${ifile#$staging_dir}' from '$old_rpath' to '$new_rpath'"
			fi
		fi
	done
	if [ $nb_rm -gt 0 ]; then
		if [ -n "$lock" ]; then
			silent_run_locked "$lock" rm -f "${to_rm[@]}"
		else
			silent_run rm -f "${to_rm[@]}"
		fi
	fi
}
