#
# Common utilities for recipes using Python 3
#

if ${configure_host_prefix}pkg-config --exists python3; then
	PYTHON_VERSION=$(${configure_host_prefix}pkg-config --modversion python3)
	PYTHON_DIR=python${PYTHON_VERSION}
	PYTHON_SITEPACKAGES_DIR=$libdir/${PYTHON_DIR}/site-packages
fi

#
# Not finding Python3 is only an error once the Configure task was executed:
#	*	Before Configure, there should be no build-time dependencies, except 
#		perhaps on git, svn, patch, etc. but these are either build-system
#		prerequisites or explicitly generated in advance by some ad-hoc 
#		mechanism when creating the native toolset
#	*	Configure itself will be able to check if Python3 is here and provide
#		an appropriate error message, failing if you really need it and don't
#		find it
# After that it should not really matter: if Configure find it, we should
# find it too, otherwise either the Configure task failed and we do not run 
# any other task, either it succeeds but should have configured the package
# to work although Python3 was not found.
#

if $recipe_is_native || $recipe_is_cross; then
	if [ -n "$PYTHON_VERSION" ]; then
		BUILD_PYTHON_VERSION=$PYTHON_VERSION
		BUILD_PYTHON_DIR=$PYTHON_DIR
		BUILD_PYTHON_SITEPACKAGES_DIR=$PYTHON_SITEPACKAGES_DIR
	elif [ $current_task != "fetch" ] && \
	     [ $current_task != "import" ] && \
         [ -r $(recipe_task_state_file configure) ]
	then
		fatal 1 "Package '$recipe_id' uses Python3, but Python3 is not available in the native sysroot"
	fi
else
	if ${configure_build_prefix}pkg-config --exists python3; then
		BUILD_PYTHON_VERSION=$(${configure_build_prefix}pkg-config --modversion python3)
		BUILD_PYTHON_DIR=python${BUILD_PYTHON_VERSION}
		BUILD_PYTHON_SITEPACKAGES_DIR=$libdir/${BUILD_PYTHON_DIR}/site-packages
	fi
	if [ -z "$PYTHON_VERSION" ] && \
	   [ -z "$BUILD_PYTHON_VERSION" ] && \
	   [ $current_task != "fetch" ] && \
	   [ $current_task != "import" ] && \
       [ -r $(recipe_task_state_file configure) ]
	then
		fatal 1 "Package '$recipe_id' uses Python3, but Python3 is not available, neither in the native sysroot nor in the target sysroot"
	fi
fi
