######################################################################################################################
#                                                                                                                    #
#                                             FAT Rootfs utilities                                            #
#                                                                                                                    #
######################################################################################################################


##################################################
# Generate the FAT image using mkfs.vfat
# -n volume-name
# -S logical-sector-size
# -F fat-size
# -C output file + size
# fsck.vfat -na
##################################################
fatfs_generate() {
	local imgRoot="$1"
	local imgType="${2:-vfat}"
	local imgName="$3"
	local partName="$4"
	local defaultSize=$(du -s $imgRoot | cut -f1)
	defaultSize=$(expr ${defaultSize}*1.3 | bc)
	defaultSize=${defaultSize%%.*}
	
	local imgSize="${5:-$defaultSize}"
	
	if [ -z "$imgSize" ] || [ $imgSize -eq 0 ]; then
		imgSize=$defaultSize
	fi

	# If generating an empty image the size of the sparse block should be large
	# enough to allocate an ext4 filesystem using 4096 bytes per inode, this is
	# about 60K, so dd needs a minimum count of 60, with bs=1024 (bytes per IO)
	eval local COUNT="0"
	eval local MIN_COUNT="60"
	if [ $imgSize -lt $MIN_COUNT ]; then
		eval COUNT="$MIN_COUNT"
	fi
	
	
	
	# Ensure total sectors is an integral number of sectors per
	# track or mcopy will complain. Sectors are 512 bytes, and and
	# we generate images with 32 sectors per track. This calculation
	# is done in blocks, which are twice the size of sectors, thus
	# the 16 instead of 32.
	BLOCK_SIZE=$imgSize
	if [ $(expr $imgSize % 16) -gt 0 ]; then
		BLOCK_SIZE=$(expr $imgSize + $(expr 16 - $(expr $imgSize % 16)))
	fi

	# Create a sparse image block
	log_exec mkfs.vfat -n ${partName} -S 512 -F 32 -C ${imagesDir}/${imgName}.vfat ${BLOCK_SIZE}
	
	# Copy files into partition
	log_exec mcopy -i ${imagesDir}/${imgName}.vfat -s ${imgRoot}/* ::/
	
	# Error codes 0 indicate successfull operation of fsck (no errors or errors corrected)
	log_exec "fsck.vfat -na ${imagesDir}/${imgName}.vfat"
}
