######################################################################################################################
#                                                                                                                    #
#                                                   Rootfs utilities                                                 #
#                                                                                                                    #
######################################################################################################################

###########################################################################
#
# Load the ROOTFS-specific script then generate rootfs
#
# This script must define one function, called with the following parameters
#
#	generate_rootfs
#		imgRoot			Directory where the partition content is located
#		imgType			Partition format
#		imgName			Output Image name
#		partName		Partition name (required for FAT)
#		imgSize			Output Image size (optional)
#
###########################################################################
generate_rootfs() {
	local imgRoot="$1"
	local imgType="$2"
	local imgName="$3"
	local partName="$4"
	local imgSize="$5"
	local fs_tools
	
	case $imgType in
		tar*)		# Generate a global tarball image
			fs_tool="tarfs"
			;;
		ext[234])	# Generate a global EXTn image
			fs_tool="extfs"
			;;
		vfat|fat32)		# Generate a DOS FAT volumn image
			fs_tool="fatfs"
			;;
		*)
			fs_tool=$imgType
			;;
	esac
	
	# Load the script
	# Launch generation command
	if [ -n "$fs_tool" ]; then
		fs_script=$(search_script rootfs/${fs_tool}_utils.sh)
		if [ -z "$fs_script" ]; then
			fatal 1 "Don't find handler to generate rootfs from the $fs_tool Rootfs Utility"
		fi
		load_script_once "$fs_script"
		${fs_tool}_generate $imgRoot $imgType $imgName $partName $imgSize
	fi
}




