##############################################################################
#                                                                            #
#                              Generic utilities                             #
#                                                                            #
##############################################################################

#
# Before anything, switch to common "C" language locale to avoid localization uglinesses
# If at any moment we really need local language support, we could restore these
#
ORIGINAL_LOCALE_LANG="$LANG";           export ORIGINAL_LOCALE_LANG
ORIGINAL_LOCALE_LANGUAGE="$LANGUAGE";   export ORIGINAL_LOCALE_LANGUAGE
ORIGINAL_LOCALE_LC_ALL="$LC_ALL";       export ORIGINAL_LOCALE_LC_ALL
ORIGINAL_LOCALE_LC_CTYPE="$LC_CTYPE";   export ORIGINAL_LOCALE_LC_CTYPE
LANG=C;                                 export LANG
LANGUAGE=C;                             export LANGUAGE
LC_ALL=C;                               export LC_ALL
LC_CTYPE=C;                             export LC_CTYPE

here=$(pwd)
# script_lib_dir is defined by Eclipse, but not the Eclipse install base directory
sw4l_install_base=${script_lib_dir%/plugins/fr.ac6.*}

#
# Canonicalize a path to an absolute path, even if the target does not yet exists
#
absolute_path() {
	readlink -m $1
}

#
# Trim a string
#
trim() {
	local var="$*"
	# remove leading whitespace characters
	var="${var#"${var%%[![:space:]]*}"}"
	# remove trailing whitespace characters
	var="${var%"${var##*[![:space:]]}"}"   
	echo -n "$var"
}

#
# Get the file name of the currently executing shell code
#
# Parameter is the level in call stack from where we want to get the file name
# Passing 0 (or nothing) will return the filename of the call, 1 the one of its
# caller (assuming the call is done in a function) a.s.o
current_script_file() {
	local depth=${1:-0}
    local sourced="${BASH_SOURCE[$depth + 1]}"
	# BASH_SOURCE contains only absolute path names (even if accessed through the PATH)
	echo "$sourced"
}

#
# Record an action to execute on script cmeanup
#
# This extend the "trap 'command' EXIT" to allow to accumulate cleanup commands
#
declare -g -a exit_cleanups # This script may be sourced in a function; without -g the variable would be local
nb_exit_cleanups=0

record_cleanup() {
	local command="$*"
	exit_cleanups[nb_exit_cleanups++]="$command"
}

execute_cleanups() {
	local cleanup=$nb_exit_cleanups
	while [ $cleanup -ge 0 ]; do
		local command=${exit_cleanups[cleanup--]}
		eval $command
	done
	unset exit_cleanups
	declare -g -a exit_cleanups
	nb_exit_cleanups=0
}
trap execute_cleanups EXIT

# Now load the log and recipe utilities from either the platform "scripts" directory
# or the current directory

if [ -r "$PLATFORM_LOC/scripts/log_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/log_utils.sh"
elif [ -r "$PLATFORM_LOC/scripts/plugins/linux.utils/log_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/plugins/linux.utils/log_utils.sh"
else
	source "$script_lib_dir/log_utils.sh"
fi

if [ -r "$PLATFORM_LOC/scripts/recipe_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/recipe_utils.sh"
elif [ -r "$PLATFORM_LOC/scripts/plugins/linux.utils/recipe_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/plugins/linux.utils/recipe_utils.sh"
else
	source "$script_lib_dir/recipe_utils.sh"
fi

# Check if System Workbench is installed on an proprietary IDE 
# If needed, run compatibility script to set/unset variables

if [ -r "$PLATFORM_LOC/scripts/compatibility_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/compatibility_utils.sh"
elif [ -r "$PLATFORM_LOC/scripts/plugins/linux.utils/compatibility_utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/plugins/linux.utils/compatibility_utils.sh"
else
	source "$script_lib_dir/compatibility_utils.sh"
fi

true # call above may return 1
